# Contributing

1. Install node. It's recommended to install `nvm` to switch between node versions.
2. Execute npm in root project folder `npm install` will install all needed packages in `node_modules` folder.
3. Execute `npm run dev` to start watching SCSS and JS files. The demo file will then automatically be served up by [Browsersync](https://browsersync.io/). All changes made will automatically be watched and the page live reloaded when changes are made.

## npm tasks
Everything should be handled by npm. Note that you don't need to interact with Gulp in a direct way.

* **`npm run dev`**: Start watching SCSS & JS files along with the demo index.html. This project uses [Browsersync](https://browsersync.io/).
* **`npm run build`**: Compiles SCSS & JS then quits.

## Workflow

Do not commit on `development` or `master` branches. `development` is the code that will be used for beta testing. `master` should always be synced with the latest version.

- Create a new branch from `development` branch: `git checkout -b branch-name`. Try to give it a descriptive name. For example:
    * `new/modal-box` for new features
    * `enhance/floating-notification` for enhancements
    * `fix/header-padding-incorrect` for bugfixing
- Make your commits and push the new branch: `git push -u origin branch-name`
- File the new Pull Request against `development` branch
- Assign somebody to review your code.
- Once the PR is approved and finished, merge it in `development` branch.
- Delete your branch locally and make sure that it does not longer exist remote.

It's a good idea to create the Pull Request as soon as possible so everybody knows what's going on with the project from the PRs screen in Bitbucket.

## Body Class

In order to avoid conflicts with other versions of this library, we are using a versioned body class in the format `.sui-x-x-x`. This class is automatically generated by the `body-class` mixin which should wrap all scss in the project.

## Updating Shared UI Package

### Versioning

Follow semantic versioning [http://semver.org/](http://semver.org/) as `package.json` won't work otherwise.

- `X.X.0` for major versions
- `X.X.X` for minor versions
- `X.X[.X||.0]-rc.1` for release candidates
- `X.X[.X||.0]-beta.1` for betas
- `X.X[.X||.0]-alpha.1` for alphas

### NPM

```
npm version <version number>
```

This will tag the release and update the package.json version. See [How to update a package](https://docs.npmjs.com/getting-started/publishing-npm-packages#how-to-update-a-package)

Ensure version/body-class are all updated in the following locations:

+ `package.json` (Should already be updated from running the previous command)
+ `scss/_variables.scss`
+ `index.html` (the body class in the final format .sui-x-x-x)

In order for the compiled JS to have the proper version number taken from package.json, we must run our build command:

```
npm run build
```

Finally we can publish the package:

```
npm publish
```
